# coding: utf-8
from __future__ import absolute_import, unicode_literals

from aio_client.provider.api import PostProviderRequest, push_request
from concentrator.smev3 import kinder_conc
from concentrator.smev3.base.utils import CustomStringIO, get_oktmo_region

from .steps import CheckOrderStep, Response


def process(request_declaration):
    """Запуск шагов проверки заявления и возвращение ответа."""

    next_step = CheckOrderStep(request_declaration)
    result = None
    while result is None:
        next_step = next_step.next()
        if isinstance(next_step, Response):
            result = next_step

    response = kinder_conc.FormDataResponseType(
        oktmo=get_oktmo_region(),
        changeOrderInfo=kinder_conc.changeOrderInfoType(
            orderId=kinder_conc.orderIdType(result.order_id),
            statusCode=kinder_conc.statusCodeType(result.status_code),
            comment=result.comment,
    ))

    mystdout = CustomStringIO()
    response.export(mystdout, 0, name_='FormDataResponse', pretty_print=False)
    return mystdout.getvalue()


def application_request_helper(*args, **kwargs):
    """Разбор входящего пакета и отправка ответа."""

    assert 'message' in kwargs
    message = kwargs.get('message')

    body = message['body']
    request_body = kinder_conc.parseString(body, silence=True)

    response = push_request(PostProviderRequest(
        origin_message_id=message['origin_message_id'],
        body=process(request_body.ApplicationRequest),
        message_type=message['message_type'],
        attachments='',
        content_failure_code=None,
        replay_to=message['replay_to'],
        is_test_message=message['is_test_message']
    ))

    return response
