# coding: utf-8

from aio_client.provider.api import PostProviderRequest, push_request
from concentrator.smev3 import kinder_conc
from concentrator.smev3.base.utils import (CustomStringIO,
                                           get_declaration_units,
                                           get_oktmo_region,
                                           get_queue_info_short, get_spec_decl)
from kinder.webservice.api.declaration import get_decl_by_client_id


def get_application_queue_helper(*args, **kwargs):
    """Разбор входящего пакета и отправка ответа
    :param message:
    :return:
    """
    assert 'message' in kwargs
    message = kwargs.get('message')

    body = message['body']
    request_body = kinder_conc.parseString(body)

    declaration = get_decl_by_client_id(
        request_body.GetApplicationQueueRequest.orderId)
    get_application_response_message = get_application_queue_response(
        declaration)

    response = push_request(PostProviderRequest(
        origin_message_id=message["origin_message_id"],
        body=get_application_response_message,
        message_type=message["message_type"],
        attachments='',
        content_failure_code=None,
        replay_to=message["replay_to"],
        is_test_message=message["is_test_message"]
    ))

    return response


def get_application_queue_response(declaration):
    """формируем тип на отправку"""
    decl_units = get_declaration_units(declaration)
    get_app_queue_type = kinder_conc.GetApplicationQueueResponseType(
            orderId=int(declaration.client_id),
            EntryDate=declaration.desired_date,
            AdaptationGroup=get_spec_decl(declaration),
            ScheduleType=getattr(declaration, 'work_type__id', None))
    for _, unit_id, _ in decl_units:
        number, all_queue = get_queue_info_short(declaration, unit_id)
        edu_org_queue_type = kinder_conc.EduOrganizationQueueType(
            Code=str(unit_id),
            NumberInQueue=number,
            AllInQueue=all_queue,
        )
        get_app_queue_type.add_EduOrganizationQueue(edu_org_queue_type)

    response = kinder_conc.FormDataResponseType(
        oktmo=get_oktmo_region(),
        GetApplicationQueueResponse=get_app_queue_type)

    mystdout = CustomStringIO()
    response.export(mystdout, 0, name_='FormDataResponse', pretty_print=False)
    return mystdout.getvalue()
